@@ -10,15 +10,16 @@ def run_classification_multiobj_example(with_plot=True, timeout=None):
     del test_data['class']
     problem = 'classification'
 
-    auto_model = Fedot(problem=problem, timeout=timeout, preset='best_quality',
-                       composer_params={'metric': ['f1', 'node_num'],
-                                        'with_tuning': False}, seed=42)
+    metric_names = ['f1', 'node_num']
+    auto_model = Fedot(problem=problem, timeout=timeout, preset='best_quality', seed=42,
+                       metric=metric_names,
+                       with_tuning=False)
     auto_model.fit(features=train_data, target='class')
     prediction = auto_model.predict_proba(features=test_data)
     print(auto_model.get_metrics(target))
     auto_model.plot_prediction()
     if with_plot:
-        auto_model.best_models.show()
+        auto_model.plot_pareto()
 
     return prediction
 
