@@ -4,13 +4,19 @@ from copy import copy, deepcopy
 from dataclasses import dataclass, field
 from typing import List, Optional, Tuple, Union
 
-import imageio
 import numpy as np
 import pandas as pd
-from PIL import Image
 
+from fedot.utilities.requirements_notificator import warn_requirement
+
+try:
+    import cv2
+except ModuleNotFoundError:
+    warn_requirement('opencv-python')
+    cv2 = None
+
+from fedot.core.data.array_utilities import atleast_2d
 from fedot.core.data.load_data import JSONBatchLoader, TextBatchLoader
-from fedot.core.data.merge import DataMerger
 from fedot.core.data.supplementary_data import SupplementaryData
 from fedot.core.repository.dataset_types import DataTypesEnum
 from fedot.core.repository.tasks import Task, TaskTypesEnum
@@ -21,10 +27,13 @@ class Data:
     """
     Base Data type class
     """
+
     idx: np.array
-    features: np.array
     task: Task
     data_type: DataTypesEnum
+    features: np.array
+    target: Optional[np.array] = None
+
     # Object with supplementary info
     supplementary_data: SupplementaryData = field(default_factory=SupplementaryData)
 
@@ -34,32 +43,30 @@ class Data:
                  task: Task = Task(TaskTypesEnum.classification),
                  data_type: DataTypesEnum = DataTypesEnum.table,
                  columns_to_drop: Optional[List] = None,
-                 target_columns: Union[str, List] = ''):
-        """
-        :param file_path: the path to the CSV with data
-        :param columns_to_drop: the names of columns that should be dropped
-        :param delimiter: the delimiter to separate the columns
-        :param task: the task that should be solved with data
-        :param data_type: the type of data interpretation
-        :param target_columns: name of target column (last column if empty and no target if None)
-        :return:
+                 target_columns: Union[str, List] = '',
+                 index_col: Optional[Union[str, int]] = 0):
+        """Import data from ``csv``
+
+        Args:
+            file_path: the path to the ``CSV`` with data
+            columns_to_drop: the names of columns that should be dropped
+            delimiter: the delimiter to separate the columns
+            task: the :obj:`Task` that should be solved with data
+            data_type: the type of data interpretation
+            target_columns: name of target column (last column if empty and no target if ``None``)
+            index_col: column name or index to use as the :obj:`Data.idx`;\n
+                if ``None`` then arrange new unique index
+
+        Returns:
+            data
         """
 
-        data_frame = pd.read_csv(file_path, sep=delimiter)
+        data_frame = pd.read_csv(file_path, sep=delimiter, index_col=index_col)
         if columns_to_drop:
             data_frame = data_frame.drop(columns_to_drop, axis=1)
 
-        # Get indices of the DataFrame
-        data_array = np.array(data_frame).T
-        idx = data_array[0]
-        if isinstance(idx[0], float) and idx[0] == round(idx[0]):
-            # if float indices is unnecessary
-            idx = [str(round(i)) for i in idx]
-        if type(target_columns) is list:
-            features, target = process_multiple_columns(target_columns, data_frame)
-        else:
-            features, target = process_one_column(target_columns, data_frame,
-                                                  data_array)
+        idx = data_frame.index.to_numpy()
+        features, target = process_target_and_features(data_frame, target_columns)
 
         return InputData(idx=idx, features=features, target=target, task=task, data_type=data_type)
 
@@ -107,15 +114,20 @@ class Data:
                                    columns_to_use: Optional[list] = None,
                                    target_column: Optional[str] = ''):
         """
-        Forms InputData of multi_ts type from columns of different variant of the same variable
+        Forms :obj:`InputData` of ``multi_ts`` type from columns of different variant of the same variable
 
-        :param task: the task that should be solved with data
-        :param file_path: path to csv file
-        :param delimiter: delimiter for pandas df
-        :param is_predict: is preparing for fit or predict stage
-        :param columns_to_use: list with names of columns of different variant of the same variable
-        :param target_column: string with name of target column, used for predict stage
+        Args:
+            task: the :obj:`Task` that should be solved with data
+            file_path: path to csv file
+            delimiter: delimiter for pandas df
+            is_predict: is preparing for fit or predict stage
+            columns_to_use: ``list`` with names of columns of different variant of the same variable
+            target_column: ``string`` with name of target column, used for predict stage
+
+        Returns:
+            data
         """
+
         df = pd.read_csv(file_path, sep=delimiter)
 
         idx = get_indices_from_file(df, file_path)
@@ -149,19 +161,25 @@ class Data:
 
         return input_data
 
-
     @staticmethod
     def from_image(images: Union[str, np.ndarray] = None,
                    labels: Union[str, np.ndarray] = None,
                    task: Task = Task(TaskTypesEnum.classification),
                    target_size: Optional[Tuple[int, int]] = None):
+        """Input data from Image
+
+        Args:
+            images: the path to the directory with image data in ``np.ndarray`` format
+                or array in ``np.ndarray`` format
+            labels: the path to the directory with image labels in ``np.ndarray`` format
+                or array in ``np.ndarray`` format
+            task: the :obj:`Task` that should be solved with data
+            target_size: size for the images resizing (if necessary)
+
+        Returns:
+            data
         """
-        :param images: the path to the directory with image data in np.ndarray format or array in np.ndarray format
-        :param labels: the path to the directory with image labels in np.ndarray format or array in np.ndarray format
-        :param task: the task that should be solved with data
-        :param target_size: size for the images resizing (if necessary)
-        :return:
-        """
+
         features = images
         target = labels
 
@@ -183,6 +201,9 @@ class Data:
                 # upload from array
                 features = np.load(images)
                 target = np.load(labels)
+                # add channels if None
+                if len(features.shape) == 3:
+                    features = np.expand_dims(features, -1)
 
         idx = np.arange(0, len(features))
 
@@ -232,36 +253,44 @@ class Data:
                         label: str = 'label',
                         task: Task = Task(TaskTypesEnum.classification),
                         data_type: DataTypesEnum = DataTypesEnum.table,
-                        export_to_meta=False, is_multilabel=False) -> 'InputData':
-        """
-        Generates InputData from the set of JSON files with different fields
-        :param files_path: path the folder with jsons
-        :param fields_to_use: list of fields that will be considered as a features
-        :param label: name of field with target variable
-        :param task: task to solve
-        :param data_type: data type in fields (as well as type for obtained InputData)
-        :param export_to_meta: combine extracted field and save to CSV
-        :param is_multilabel: if True, creates multilabel target
-        :return: combined dataset
+                        export_to_meta=False, is_multilabel=False, shuffle=True) -> 'InputData':
+        """Generates InputData from the set of ``JSON`` files with different fields
+
+        Args:
+            files_path: path the folder with ``json`` files
+            fields_to_use: ``list`` of fields that will be considered as a features
+            label: name of field with target variable
+            task: :obj:`Task` to solve
+            data_type: data type in fields (as well as type for obtained :obj:`InputData`)
+            export_to_meta: combine extracted field and save to ``CSV``
+            is_multilabel: if ``True``, creates multilabel target
+            shuffle: if ``True``, shuffles data
+
+        Returns:
+            combined dataset
         """
 
         if os.path.isfile(files_path):
             raise ValueError("""Path to the directory expected but got file""")
 
-        df_data = JSONBatchLoader(path=files_path, label=label, fields_to_use=fields_to_use).extract(export_to_meta)
+        df_data = JSONBatchLoader(path=files_path, label=label, fields_to_use=fields_to_use,
+                                  shuffle=shuffle).extract(export_to_meta)
 
         if len(fields_to_use) > 1:
             fields_to_combine = []
-            for f in fields_to_use:
-                fields_to_combine.append(np.array(df_data[f]))
+            for field in fields_to_use:
+                fields_to_combine.append(np.array(df_data[field]))
+                # Unite if the element of text data is divided into strings
+                if isinstance(df_data[field][0], list):
+                    df_data[field] = [' '.join(piece) for piece in df_data[field]]
 
             features = np.column_stack(tuple(fields_to_combine))
         else:
-            val = df_data[fields_to_use[0]]
+            field = df_data[fields_to_use[0]]
             # process field with nested list
-            if isinstance(val[0], list):
-                val = [' '.join(v) for v in val]
-            features = np.array(val)
+            if isinstance(field[0], list):
+                field = [' '.join(piece) for piece in field]
+            features = np.array(field)
 
         if is_multilabel:
             target = df_data[label]
@@ -284,32 +313,30 @@ class Data:
         return InputData(idx=idx, features=features,
                          target=target, task=task, data_type=data_type)
 
+    def to_csv(self, path_to_save):
+        dataframe = pd.DataFrame(data=self.features, index=self.idx)
+        if self.target is not None:
+            dataframe['target'] = self.target
+        dataframe.to_csv(path_to_save)
+
 
 @dataclass
 class InputData(Data):
+    """Data class for input data for the nodes
     """
-    Data class for input data for the nodes
-    """
-    target: Optional[np.array] = None
 
     @property
     def num_classes(self) -> Optional[int]:
+        unique_values = self.class_labels
+        return len(unique_values) if unique_values is not None else None
+
+    @property
+    def class_labels(self) -> Optional[int]:
         if self.task.task_type == TaskTypesEnum.classification and self.target is not None:
-            return len(np.unique(self.target))
+            return np.unique(self.target)
         else:
             return None
 
-    @staticmethod
-    def from_predictions(outputs: List['OutputData']):
-        """ Method obtain predictions from previous nodes """
-        # Update not only features but idx, target and task also
-        idx, features, target, task, d_type, updated_info = DataMerger(outputs).merge()
-
-        # Mark data as preprocessed already
-        updated_info.was_preprocessed = True
-        return InputData(idx=idx, features=features, target=target, task=task,
-                         data_type=d_type, supplementary_data=updated_info)
-
     def subset_range(self, start: int, end: int):
         if not (0 <= start <= end <= len(self.idx)):
             raise ValueError('Incorrect boundaries for subset')
@@ -321,11 +348,15 @@ class InputData(Data):
                          task=self.task, data_type=self.data_type)
 
     def subset_indices(self, selected_idx: List):
+        """Get subset from :obj:`InputData` to extract all items with specified indices
+
+        Args:
+            selected_idx: ``list`` of indices for extraction
+
+        Returns:
+            :obj:`InputData`
         """
-        Get subset from InputData to extract all items with specified indices
-        :param selected_idx: list of indices for extraction
-        :return:
-        """
+
         idx_list = [str(i) for i in self.idx]
 
         # extractions of row number for each existing index from selected_idx
@@ -340,7 +371,9 @@ class InputData(Data):
                          task=self.task, data_type=self.data_type)
 
     def subset_features(self, features_ids: list):
-        """ Return new InputData with subset of features based on features_ids list """
+        """Return new :obj:`InputData` with subset of features based on ``features_ids`` list
+        """
+
         subsample_features = self.features[:, features_ids]
         subsample_input = InputData(features=subsample_features,
                                     data_type=self.data_type,
@@ -351,10 +384,10 @@ class InputData(Data):
         return subsample_input
 
     def shuffle(self):
+        """Shuffles features and target if possible
         """
-        Shuffles features and target if possible
-        """
-        if self.data_type == DataTypesEnum.table:
+
+        if self.data_type in (DataTypesEnum.table, DataTypesEnum.image, DataTypesEnum.text):
             shuffled_ind = np.random.permutation(len(self.features))
             idx, features, target = np.asarray(self.idx)[shuffled_ind], self.features[shuffled_ind], self.target[
                 shuffled_ind]
@@ -365,7 +398,9 @@ class InputData(Data):
             pass
 
     def convert_non_int_indexes_for_fit(self, pipeline):
-        """ Conversion non int (datetime, string, etc) indexes in integer form in fit stage """
+        """Conversion non ``int`` (``datetime``, ``string``, etc) indexes in ``integer`` form on the fit stage
+        """
+
         copied_data = deepcopy(self)
         is_timestamp = isinstance(copied_data.idx[0], pd._libs.tslibs.timestamps.Timestamp)
         is_numpy_datetime = isinstance(copied_data.idx[0], np.datetime64)
@@ -380,14 +415,16 @@ class InputData(Data):
             pipeline.last_idx_int = copied_data.idx[-1]
             pipeline.last_idx_dt = last_idx_time
             pipeline.period = last_idx_time - pre_last_time
-        elif type(copied_data.idx[0]) not in [int, np.int32, np.int64]:
+        elif not isinstance(copied_data.idx[0], (int, np.int32, np.int64)):
             copied_data.supplementary_data.non_int_idx = copy(copied_data.idx)
             copied_data.idx = np.array(range(len(copied_data.idx)))
             pipeline.last_idx_int = copied_data.idx[-1]
         return copied_data
 
     def convert_non_int_indexes_for_predict(self, pipeline):
-        """Conversion non int (datetime, string, etc) indexes in integer form in predict stage"""
+        """Conversion non ``int`` (``datetime``, ``string``, etc) indexes in ``integer`` form on the predict stage
+        """
+
         copied_data = deepcopy(self)
         is_timestamp = isinstance(copied_data.idx[0], pd._libs.tslibs.timestamps.Timestamp)
         is_numpy_datetime = isinstance(copied_data.idx[0], np.datetime64)
@@ -395,11 +432,11 @@ class InputData(Data):
         if is_timestamp or is_numpy_datetime:
             copied_data.supplementary_data.non_int_idx = copy(self.idx)
             copied_data.idx = self._resolve_non_int_idx(pipeline)
-        elif type(copied_data.idx[0]) not in [int, np.int32, np.int64]:
+        elif not isinstance(copied_data.idx[0], (int, np.int32, np.int64)):
             # note, that string indexes do not have an order and always we think that indexes we want to predict go
             # immediately after the train indexes
             copied_data.supplementary_data.non_int_idx = copy(copied_data.idx)
-            copied_data.idx = pipeline.last_idx_int + np.array(range(1, len(copied_data.idx)+1))
+            copied_data.idx = pipeline.last_idx_int + np.array(range(1, len(copied_data.idx) + 1))
         return copied_data
 
     @staticmethod
@@ -412,71 +449,70 @@ class InputData(Data):
 
 @dataclass
 class OutputData(Data):
+    """``Data`` type for data prediction in the node
     """
-    Data type for data prediction in the node
+
+    features: Optional[np.ndarray] = None
+    predict: np.ndarray = None
+    target: Optional[np.ndarray] = None
+
+
+def _resize_image(file_path: str, target_size: Tuple[int, int]):
+    """Function resizes and rewrites the input image
     """
-    predict: np.array = None
-    target: Optional[np.array] = None
 
-
-def _resize_image(file_path: str, target_size: tuple):
-    im = Image.open(file_path)
-    im_resized = im.resize(target_size, Image.NEAREST)
-    im_resized.save(file_path, 'jpeg')
-
-    img = np.asarray(imageio.imread(file_path, 'jpeg'))
-    if len(img.shape) == 3:
-        # TODO refactor for multi-color
-        img = img[..., 0] + img[..., 1] + img[..., 2]
+    img = cv2.imread(file_path)
+    if img.shape[:2] != target_size:
+        img = cv2.resize(img, (target_size[0], target_size[1]))
+        cv2.imwrite(file_path, img)
     return img
 
 
-def process_one_column(target_column, data_frame, data_array):
-    """ Function process pandas dataframe with single column
+def process_target_and_features(data_frame: pd.DataFrame,
+                                target_column: Optional[Union[str, List[str]]]
+                                ) -> Tuple[np.ndarray, Optional[np.ndarray]]:
+    """Function process pandas ``dataframe`` with single column
 
-    :param target_column: name of column with target or None
-    :param data_frame: loaded panda DataFrame
-    :param data_array: array received from source DataFrame
-    :return features: numpy array (table) with features
-    :return target: numpy array (column) with target
+    Args:
+        data_frame: loaded pandas :obj:`DataFrame`
+        target_column: names of columns with target or ``None``
+
+    Returns:
+        (``np.array`` (table) with features, ``np.array`` (column) with target)
     """
+
     if target_column == '':
         # Take the last column in the table
         target_column = data_frame.columns[-1]
 
-    if target_column and target_column in data_frame.columns:
-        target = np.array(data_frame[target_column])
-        pos = list(data_frame.keys()).index(target_column)
-        features = np.delete(data_array.T, [0, pos], axis=1)
+    if target_column:
+        target = atleast_2d(data_frame[target_column].to_numpy())
+        features = data_frame.drop(columns=target_column).to_numpy()
     else:
-        # no target in data
-        features = data_array[1:].T
         target = None
-
-    if target is not None:
-        target = np.array(target)
-        if len(target.shape) < 2:
-            target = target.reshape((-1, 1))
+        features = data_frame.to_numpy()
 
     return features, target
 
 
-def process_multiple_columns(target_columns, data_frame):
-    """ Function for processing target """
-    features = np.array(data_frame.drop(columns=target_columns))
-
-    # Remove index column
-    targets = np.array(data_frame[target_columns])
-
-    return features, targets
-
-
 def data_type_is_table(data: Union[InputData, OutputData]) -> bool:
-    return data.data_type == DataTypesEnum.table
+    return data.data_type is DataTypesEnum.table
 
 
 def data_type_is_ts(data: InputData) -> bool:
-    return data.data_type == DataTypesEnum.ts
+    return data.data_type is DataTypesEnum.ts
+
+
+def data_type_is_multi_ts(data: InputData) -> bool:
+    return data.data_type is DataTypesEnum.multi_ts
+
+
+def data_type_is_text(data: InputData) -> bool:
+    return data.data_type is DataTypesEnum.text
+
+
+def data_type_is_image(data: InputData) -> bool:
+    return data.data_type is DataTypesEnum.image
 
 
 def get_indices_from_file(data_frame, file_path):
@@ -491,12 +527,12 @@ def get_indices_from_file(data_frame, file_path):
 def array_to_input_data(features_array: np.array,
                         target_array: np.array,
                         idx: Optional[np.array] = None,
-                        task: Task = Task(TaskTypesEnum.classification)):
-    data_type = autodetect_data_type(task)
-
+                        task: Task = Task(TaskTypesEnum.classification),
+                        data_type: Optional[DataTypesEnum] = None):
     if idx is None:
         idx = np.arange(len(features_array))
-
+    if data_type is None:
+        data_type = autodetect_data_type(task)
     return InputData(idx=idx, features=features_array, target=target_array, task=task, data_type=data_type)
 
 
