@@ -1,42 +1,41 @@
 from abc import abstractmethod
-from typing import Union
+from typing import Optional, Union
 
 from fedot.core.data.data import InputData, OutputData
-from fedot.core.log import Log, default_log
-from fedot.core.operations.warnings_processor import suppress_stdout
+from fedot.core.log import default_log
+from fedot.core.operations.hyperparameters_preprocessing import HyperparametersPreprocessor
+from fedot.core.operations.operation_parameters import OperationParameters
 from fedot.core.repository.operation_types_repository import OperationMetaInfo
 from fedot.core.repository.tasks import Task, TaskTypesEnum, compatible_task_types
-from fedot.core.utils import DEFAULT_PARAMS_STUB
 
 
 class Operation:
-    """
-    Base class for operations in nodes. Operations could be machine learning
+    """Base class for operations in nodes. Operations could be machine learning
     (or statistical) models or data operations
 
-    :param operation_type: name of the operation
-    :param log: Log object to record messages
+    Args:
+        operation_type: name of the operation
     """
 
-    def __init__(self, operation_type: str, log: Log = None, **kwargs):
+    def __init__(self, operation_type: str, **kwargs):
         self.operation_type = operation_type
-        self.log = log
 
         self._eval_strategy = None
         self.operations_repo = None
         self.fitted_operation = None
 
-        if not log:
-            self.log = default_log(__name__)
-        else:
-            self.log = log
+        self.log = default_log(self)
 
     def _init(self, task: Task, **kwargs):
         params = kwargs.get('params')
-        params_for_fit = None
-        if params != DEFAULT_PARAMS_STUB:
-            params_for_fit = params
-
+        if not params:
+            params = OperationParameters.from_operation_type(self.operation_type)
+        if isinstance(params, dict):
+            params = OperationParameters.from_operation_type(self.operation_type, **params)
+        params_for_fit = HyperparametersPreprocessor(operation_type=self.operation_type,
+                                                     n_samples_data=kwargs.get('n_samples_data'))\
+            .correct(params.to_dict())
+        params_for_fit = OperationParameters.from_operation_type(self.operation_type, **params_for_fit)
         try:
             self._eval_strategy = \
                 _eval_strategy_for_task(self.operation_type,
@@ -67,47 +66,68 @@ class Operation:
             raise ValueError(f'{self.__class__.__name__} {self.operation_type} not found')
         return operation_info
 
-    def fit(self, params: Union[str, dict, None], data: InputData, is_fit_pipeline_stage: bool = True):
-        """
-        This method is used for defining and running of the evaluation strategy
+    def fit(self, params: Optional[Union[OperationParameters, dict]], data: InputData):
+        """This method is used for defining and running of the evaluation strategy
         to train the operation with the data provided
 
-        :param params: hyperparameters for operation
-        :param data: data used for operation training
-        :return: tuple of trained operation and prediction on train data
-        :param is_fit_pipeline_stage: is this fit or predict stage for pipeline
+        Args:
+            params: hyperparameters for operation
+            data: data used for operation training
+
+        Returns:
+            tuple: trained operation and prediction on train data
         """
+        self._init(data.task, params=params, n_samples_data=data.features.shape[0])
 
-        self._init(data.task, params=params)
+        self.fitted_operation = self._eval_strategy.fit(train_data=data)
 
-        with suppress_stdout():
-            self.fitted_operation = self._eval_strategy.fit(train_data=data)
-
-        predict_train = self.predict(self.fitted_operation, data, is_fit_pipeline_stage, params)
+        predict_train = self.predict_for_fit(self.fitted_operation, data, params)
 
         return self.fitted_operation, predict_train
 
-    def predict(self, fitted_operation, data: InputData, is_fit_pipeline_stage: bool,
-                params: Union[str, dict, None] = None, output_mode: str = 'default'):
-        """
-        This method is used for defining and running of the evaluation strategy
+    def predict(self, fitted_operation, data: InputData, params: Optional[Union[OperationParameters, dict]] = None,
+                output_mode: str = 'default'):
+        """This method is used for defining and running of the evaluation strategy
         to predict with the data provided
 
-        :param fitted_operation: trained operation object
-        :param data: data used for prediction
-        :param is_fit_pipeline_stage: is this fit or predict stage for pipeline
-        :param params: hyperparameters for operation
-        :param output_mode: string with information about output of operation,
-        for example, is the operation predict probabilities or class labels
+        Args:
+            fitted_operation: trained operation object
+            data: data used for prediction
+            params: hyperparameters for operation
+            output_mode: string with information about output of operation,
+            for example, is the operation predict probabilities or class labels
         """
+        return self._predict(fitted_operation, data, params, output_mode, is_fit_stage=False)
+
+    def predict_for_fit(self, fitted_operation, data: InputData, params: Optional[OperationParameters] = None,
+                        output_mode: str = 'default'):
+        """This method is used for defining and running of the evaluation strategy
+        to predict with the data provided during fit stage
+
+        Args:
+            fitted_operation: trained operation object
+            data: data used for prediction
+            params: hyperparameters for operation
+            output_mode: string with information about output of operation,
+                for example, is the operation predict probabilities or class labels
+        """
+        return self._predict(fitted_operation, data, params, output_mode, is_fit_stage=True)
+
+    def _predict(self, fitted_operation, data: InputData, params: Optional[OperationParameters] = None,
+                 output_mode: str = 'default', is_fit_stage: bool = False):
+
         is_main_target = data.supplementary_data.is_main_target
         data_flow_length = data.supplementary_data.data_flow_length
-        self._init(data.task, output_mode=output_mode, params=params)
+        self._init(data.task, output_mode=output_mode, params=params, n_samples_data=data.features.shape[0])
 
-        prediction = self._eval_strategy.predict(
-            trained_operation=fitted_operation,
-            predict_data=data,
-            is_fit_pipeline_stage=is_fit_pipeline_stage)
+        if is_fit_stage:
+            prediction = self._eval_strategy.predict_for_fit(
+                trained_operation=fitted_operation,
+                predict_data=data)
+        else:
+            prediction = self._eval_strategy.predict(
+                trained_operation=fitted_operation,
+                predict_data=data)
         prediction = self.assign_tabular_column_types(prediction, output_mode)
 
         if is_main_target is False:
@@ -120,9 +140,10 @@ class Operation:
     @staticmethod
     @abstractmethod
     def assign_tabular_column_types(output_data: OutputData, output_mode: str) -> OutputData:
-        """ Assign types for columns based on task and output_mode (for classification)
+        """Assign types for columns based on task and output_mode (for classification)\n
         For example, pipeline for solving time series forecasting task contains lagged and ridge operations.
-        ts_type -> lagged -> tabular type. So, there is a need to assign column types to new data
+        ``ts_type -> lagged -> tabular type``\n
+        So, there is a need to assign column types to new data
         """
         raise NotImplementedError()
 
@@ -132,15 +153,16 @@ class Operation:
 
 def _eval_strategy_for_task(operation_type: str, current_task_type: TaskTypesEnum,
                             operations_repo):
-    """
-    The function returns the strategy for the selected operation and task type.
+    """The function returns the strategy for the selected operation and task type.
     And if it is necessary, found acceptable strategy for operation
 
-    :param operation_type: name of operation, for example, 'ridge'
-    :param current_task_type: task to solve
-    :param operations_repo: repository with operations
+    Args:
+        operation_type: name of operation, for example, ``'ridge'``
+        current_task_type: task to solve
+        operations_repo: repository with operations
 
-    :return strategy: EvaluationStrategy class for this operation
+    Returns:
+        EvaluationStrategy: ``EvaluationStrategy`` class for this operation
     """
 
     # Get acceptable task types for operation
