@@ -1,15 +1,18 @@
 import json
 import os
-import warnings
 from collections import defaultdict
 from dataclasses import dataclass
-from typing import Any, List, Optional
+from typing import Any, List, Optional, Dict, Union
 
 from fedot.core.constants import BEST_QUALITY_PRESET_NAME, AUTO_PRESET_NAME
+from fedot.core.log import default_log
+from fedot.core.optimisers.graph import OptNode
 from fedot.core.repository.dataset_types import DataTypesEnum
 from fedot.core.repository.json_evaluation import eval_field_str, eval_strategy_str, read_field
 from fedot.core.repository.tasks import Task, TaskTypesEnum
 
+AVAILABLE_REPO_NAMES = ['all', 'model', 'data_operation', 'automl']
+
 
 @dataclass
 class OperationMetaInfo:
@@ -23,12 +26,14 @@ class OperationMetaInfo:
     presets: Optional[List[str]] = None
 
     def current_strategy(self, task: TaskTypesEnum):
-        """
-        Method allows getting available processing strategies depending on the
+        """Method allows getting available processing strategies depending on the
         selected task
 
-        :param task: machine learning task (e.g. regression and classification)
-        :return : supported strategies for task
+        Args:
+            task: machine learning task (e.g. regression and classification)
+
+        Returns:
+            supported strategies for task
         """
 
         if isinstance(self.supported_strategies, dict):
@@ -47,28 +52,41 @@ def run_once(function):
 
 
 class OperationTypesRepository:
-    """ Class for connecting models and data operations with json files with
-    its descriptions and metadata"""
+    """Class for connecting models and data operations with json files with
+    its descriptions and metadata
+    """
 
     __initialized_repositories__ = {}
+    # The later the tag, the higher its priority in case of intersection
+    DEFAULT_MODEL_TAGS = ['linear', 'non_linear', 'custom_model', 'tree', 'boosting', 'ts_model', 'deep']
+    DEFAULT_DATA_OPERATION_TAGS = [
+        'data_source', 'feature_scaling', 'imputation', 'feature_reduction', 'feature_engineering', 'encoding',
+        'filtering', 'feature_selection', 'ts_to_table', 'smoothing', 'ts_to_ts', 'text', 'decompose', 'imbalanced',
+        'data_source_img', 'data_source_text', 'data_source_table', 'data_source_ts'
+    ]
 
     __repository_dict__ = {
-        'model': {'file': 'model_repository.json', 'initialized_repo': None},
-        'data_operation': {'file': 'data_operation_repository.json', 'initialized_repo': None},
-        'automl': {'file': 'automl_repository.json', 'initialized_repo': None}
+        'model': {'file': 'model_repository.json', 'initialized_repo': None, 'default_tags': DEFAULT_MODEL_TAGS},
+        'data_operation': {'file': 'data_operation_repository.json', 'initialized_repo': None,
+                           'default_tags': DEFAULT_DATA_OPERATION_TAGS},
+        'automl': {'file': 'automl_repository.json', 'initialized_repo': None, 'default_tags': []}
 
     }
 
     def __init__(self, operation_type: str = 'model'):
+        self.log = default_log(self)
+
         self._tags_excluded_by_default = ['non-default', 'expensive']
         OperationTypesRepository.init_default_repositories()
 
         self.repository_name = []
         self._repo = []
+        self.default_tags = []
         if operation_type == 'all':
-            for rep_name in OperationTypesRepository.__repository_dict__.keys():
-                self.repository_name.append(OperationTypesRepository.__repository_dict__[rep_name]['file'])
-                operations = OperationTypesRepository.__repository_dict__[rep_name]['initialized_repo']
+            for op_type in OperationTypesRepository.__repository_dict__.keys():
+                self.repository_name.append(OperationTypesRepository.__repository_dict__[op_type]['file'])
+                operations = OperationTypesRepository.__repository_dict__[op_type]['initialized_repo']
+                self.default_tags += OperationTypesRepository.__repository_dict__[op_type]['default_tags']
 
                 if operations is not None:
                     for operation in operations:
@@ -78,6 +96,7 @@ class OperationTypesRepository:
         else:
             self.repository_name = OperationTypesRepository.__repository_dict__[operation_type]['file']
             self._repo = OperationTypesRepository.__repository_dict__[operation_type]['initialized_repo']
+            self.default_tags = OperationTypesRepository.__repository_dict__[operation_type]['default_tags']
 
     @classmethod
     def get_available_repositories(cls):
@@ -130,12 +149,13 @@ class OperationTypesRepository:
 
     @classmethod
     def _initialise_repo(cls, repo_path: str) -> List[OperationMetaInfo]:
-        """ Method parse JSON repository with operations descriptions and
-        wrapped information into OperationMetaInfo, then put it into the list
+        """Method parse ``JSON`` repository with operations descriptions and
+        wrapped information into :obj:`OperationMetaInfo`, then put it into the list
 
-        :return operations_list: list with OperationMetaInfo for every operation
-        from json repository
+        Returns:
+            List[OperationMetaInfo]: list with :obj:`OperationMetaInfo` for every operation from ``json`` repository
         """
+
         repository_json = load_repository(repo_path)
 
         metadata_json = repository_json['metadata']
@@ -193,10 +213,11 @@ class OperationTypesRepository:
 
     @staticmethod
     def get_strategies_by_metadata(metadata: dict):
-        """ Method allow obtain strategy instance by the metadata
+        """Method allow obtain strategy instance by the metadata
 
-        :param metadata: information about meta of the operation
-        :return supported_strategies: available strategies for current metadata
+        Args:
+            metadata: information about meta of the operation
+            supported_strategies: available strategies for current metadata
         """
         strategies_json = metadata['strategies']
         if isinstance(strategies_json, list):
@@ -220,29 +241,37 @@ class OperationTypesRepository:
         if len(operations_with_id) > 1:
             raise ValueError('Several operations with same id in repository')
         if len(operations_with_id) == 0:
-            warnings.warn(f'Operation {operation_id} not found in the repository')
+            self.log.warning(f'Operation {operation_id} not found in the repository')
             return None
         return operations_with_id[0]
 
     def operations_with_tag(self, tags: List[str], is_full_match: bool = False):
+        """ Method returns operations from repository with specific tags
+
+                :param tags: list of required tags
+                :param is_full_match: are all tags are required
+
+                :return list of suitable operations names
+        """
         operations_info = [m for m in self._repo if
                            _is_operation_contains_tag(tags, m.tags, is_full_match)]
-        return [m.id for m in operations_info], operations_info
+        return [m.id for m in operations_info]
 
     def suitable_operation(self, task_type: TaskTypesEnum = None,
                            data_type: DataTypesEnum = None,
                            tags: List[str] = None, is_full_match: bool = False,
                            forbidden_tags: List[str] = None,
-                           preset: str = None):
-        """ Method returns operations from repository for desired task and / or
+                           preset: str = None) -> List[str]:
+        """Method returns operations from repository for desired task and / or
         tags. Filtering method.
 
-        :param task_type: task to filter
-        :param data_type: data type to filter
-        :param tags: operations with which tags are required
-        :param is_full_match: requires all tags to match, or at least one
-        :param forbidden_tags: operations with such tags shouldn't be returned
-        :param preset: return operations from desired preset
+        Args:
+            task_type: task to filter
+            data_type: data type to filter
+            tags: operations with which tags are required
+            is_full_match: requires all tags to match, or at least one
+            forbidden_tags: operations with such tags shouldn't be returned
+            preset: return operations from desired preset
         """
 
         if not forbidden_tags:
@@ -253,52 +282,100 @@ class OperationTypesRepository:
                 # Forbidden tags by default
                 forbidden_tags.append(excluded_default_tag)
 
-        if task_type is None:
-            operations_info = []
-            for o in self._repo:
-                # Perform filtering for every operation in the repository
-                tags_good = not tags or _is_operation_contains_tag(tags, o.tags, is_full_match)
-                tags_bad = not forbidden_tags or not _is_operation_contains_tag(forbidden_tags, o.tags, False)
-                is_desired_preset = _is_operation_contains_preset(o.presets, preset)
-                if tags_good and tags_bad and is_desired_preset:
-                    operations_info.append(o)
-
-        else:
-            operations_info = []
-            for o in self._repo:
-                is_desired_task = task_type in o.task_type
-                tags_good = not tags or _is_operation_contains_tag(tags, o.tags, is_full_match)
-                tags_bad = not forbidden_tags or not _is_operation_contains_tag(forbidden_tags, o.tags, False)
-                is_desired_preset = _is_operation_contains_preset(o.presets, preset)
-                if is_desired_task and tags_good and tags_bad and is_desired_preset:
-                    operations_info.append(o)
+        no_task = task_type is None
+        operations_info = []
+        for o in self._repo:
+            is_desired_task = task_type in o.task_type or no_task
+            tags_good = not tags or _is_operation_contains_tag(tags, o.tags, is_full_match)
+            tags_bad = not forbidden_tags or not _is_operation_contains_tag(forbidden_tags, o.tags, False)
+            is_desired_preset = _is_operation_contains_preset(o.presets, preset)
+            if is_desired_task and tags_good and tags_bad and is_desired_preset:
+                operations_info.append(o)
 
+        # TODO: too many operations are filtered out, because only a small number of operations defines `input_types`
         if data_type:
             operations_info = [o for o in operations_info if data_type in o.input_types]
 
-        return [m.id for m in operations_info], operations_info
+        return [m.id for m in operations_info]
 
     @property
     def operations(self):
         return self._repo
 
+    def get_first_suitable_operation_tag(self, operation: str, tags_to_find: Optional[List[str]] = None) \
+            -> Optional[str]:
+        """Finds the first suitable tag for the operation in the repository.
+
+        Args:
+            operation: name of the operation
+            tags_to_find: list of suitable tags. The later the tag, the higher its priority in case of intersection
+
+        Returns:
+            Optional[List[str]]: first suitable tag or ``None``
+        """
+
+        tags_to_find = tags_to_find or self.default_tags
+
+        info = self.operation_info_by_id(operation)
+        if info is None:
+            return None
+        for tag in reversed(tags_to_find):
+            if tag in info.tags:
+                return tag
+        return None
+
+
+def get_opt_node_tag(opt_node: Union[OptNode, str], tags_model: Optional[List[str]] = None,
+                     tags_data: Optional[List[str]] = None,
+                     repos_tags: Optional[Dict['OperationTypesRepository', List[str]]] = None) -> Optional[str]:
+    """Finds the first suitable tag for the OptNode across Fedot repositories
+
+    Args:
+        opt_node: :obj:`OptNode` or its name
+        tags_model: tags for :obj:`OperationTypesRepository('model')` to map the history operations
+            The later the tag, the higher its priority in case of intersection
+        tags_data: tags for :obj:`OperationTypesRepository('data_operation')` to map the history operations
+            The later the tag, the higher its priority in case of intersection
+        repos_tags: dictionary mapping :obj:`OperationTypesRepository` with suitable tags.
+            Can be used only if no ``tags_model`` and ``tags_data`` specified.
+    Returns:
+        Optional[str]: first suitable tag or ``None``
+    """
+
+    if (tags_model or tags_data) and repos_tags:
+        raise ValueError('Parameter repos_tags can not be set with any of these parameters: tags_model, tags_data.')
+
+    node_name = str(opt_node) if isinstance(opt_node, OptNode) else opt_node
+
+    repos_tags = repos_tags or {
+        OperationTypesRepository('model'): tags_model,
+        OperationTypesRepository('data_operation'): tags_data
+    }
+
+    for repo, tags in repos_tags.items():
+        tag = repo.get_first_suitable_operation_tag(node_name, tags)
+        if tag is not None:
+            return tag
+    return None
+
 
 def _is_operation_contains_tag(candidate_tags: List[str],
                                operation_tags: List[str],
                                is_full_match: bool) -> bool:
-    """
-    The function checks which operations are suitable for the selected tags
+    """The function checks which operations are suitable for the selected tags
 
-    :param candidate_tags: list with tags that the operation must have in order
-    to fit the selected task
-    :param operation_tags: list with tags with names as in repository json file
-    which correspond to the considering operation
-    :param is_full_match: requires all tags to match, or at least one
+    Args:
+        candidate_tags: ``list`` with tags that the operation must have in order
+            to fit the selected task
+        operation_tags: ``list`` with tags with names as in repository json file
+            which correspond to the considering operation
+        is_full_match: requires all tags to match, or at least one
 
-    :return : is there a match on the tags
+    Returns:
+        bool: is there a match on the tags
     """
 
-    matches = ([(tag in operation_tags) for tag in candidate_tags])
+    matches = (tag in operation_tags for tag in candidate_tags)
     if is_full_match:
         return all(matches)
     else:
@@ -306,7 +383,9 @@ def _is_operation_contains_tag(candidate_tags: List[str],
 
 
 def _is_operation_contains_preset(operation_presets: List[str], preset: str) -> bool:
-    """ Checking whether the operation is suitable for current preset """
+    """Checking whether the operation is suitable for current preset
+    """
+
     if preset is None:
         # None means that best_quality preset are using so return all operations
         return True
@@ -324,42 +403,37 @@ def atomized_model_meta_tags():
 
 def get_operations_for_task(task: Optional[Task], mode='all', tags=None, forbidden_tags=None,
                             preset: str = None):
-    """ Function returns aliases of operations.
+    """Function returns aliases of operations.
 
-    :param task: task to solve
-    :param mode: mode to return operations
-        The possible parameters are:
-            'all' - return list with all operations
-            'model' - return only list with models
-            'data_operation' - return only list with data_operations
-    :param tags: tags for grabbing when filtering
-    :param forbidden_tags: tags for skipping when filtering
-    :param preset: operations from this preset will be obtained
+    Args:
+        task: task to solve
+        mode: mode to return operations
 
-    :return : list with operation aliases
+            .. details:: the possible parameters of ``mode``:
+
+                - ``all`` -> return list with all operations
+                - ``model`` -> return only list with models
+                - ``data_operation`` -> return only list with data_operations
+
+        tags: tags for grabbing when filtering
+        forbidden_tags: tags for skipping when filtering
+        preset: operations from this preset will be obtained
+
+    Returns:
+        list:  operation aliases
     """
+
     # Preset None means that all operations will be returned
     if preset is not None:
         if BEST_QUALITY_PRESET_NAME in preset or AUTO_PRESET_NAME in preset:
             preset = None
 
     task_type = task.task_type if task else None
-    if mode != 'all':
+    if mode in AVAILABLE_REPO_NAMES:
         repo = OperationTypesRepository(mode)
-        model_types, _ = repo.suitable_operation(task_type, tags=tags, forbidden_tags=forbidden_tags,
-                                                 preset=preset)
+        model_types = repo.suitable_operation(task_type, tags=tags, forbidden_tags=forbidden_tags,
+                                              preset=preset)
         return model_types
-    elif mode == 'all':
-        # Get models from repository
-        repo = OperationTypesRepository('model')
-        model_types, _ = repo.suitable_operation(task_type, tags=tags, forbidden_tags=forbidden_tags,
-                                                 preset=preset)
-        # Get data operations
-        repo = OperationTypesRepository('data_operation')
-        data_operation_types, _ = repo.suitable_operation(task_type, tags=tags,
-                                                          forbidden_tags=forbidden_tags,
-                                                          preset=preset)
-        return model_types + data_operation_types
     else:
         raise ValueError(f'Such mode "{mode}" is not supported')
 
@@ -371,9 +445,12 @@ def get_operation_type_from_id(operation_id):
 
 def _operation_name_without_postfix(operation_id):
     """
-    :param operation_id: operation name with optional postfix - text after / sign
-    :return: operation type - all characters before postfix (all characters if no postfix found)
+    Args:
+        operation_id: operation name with optional postfix - text after / sign
+    Returns:
+        operation type - all characters before postfix (all characters if no postfix found)
     """
+
     postfix_sign = '/'
     # if the operation id has custom postfix
     if postfix_sign in operation_id:
