@@ -1,8 +1,10 @@
-from typing import Callable
+from numbers import Real
+from typing import Callable, Union, TypeVar
 
 from fedot.core.composer.metrics import (ComputationTime, Accuracy, F1, Logloss, MAE,
                                          MAPE, SMAPE, MSE, MSLE, Metric, NodeNum, Precision, R2,
                                          RMSE, ROCAUC, Silhouette, StructuralComplexity)
+from fedot.core.dag.graph import Graph
 from fedot.core.utilities.data_structures import ComparableEnum as Enum
 
 
@@ -10,6 +12,11 @@ class MetricsEnum(Enum):
     pass
 
 
+G = TypeVar('G', bound=Graph, covariant=True)
+MetricCallable = Callable[[G], Real]
+MetricType = Union[MetricCallable, MetricsEnum]
+
+
 class QualityMetricsEnum(MetricsEnum):
     pass
 
@@ -17,7 +24,7 @@ class QualityMetricsEnum(MetricsEnum):
 class ComplexityMetricsEnum(MetricsEnum):
     node_num = 'node_number'
     structural = 'structural'
-    computation_time = 'computation_time'
+    computation_time = 'computation_time_in_seconds'
 
 
 class ClusteringMetricsEnum(QualityMetricsEnum):
@@ -73,8 +80,8 @@ class MetricsRepository:
         ComplexityMetricsEnum.computation_time: ComputationTime.get_value
     }
 
-    def metric_by_id(self, metric_id: MetricsEnum) -> Callable:
-        return self._metrics_implementations[metric_id]
+    def metric_by_id(self, metric_id: MetricsEnum, default_callable: MetricCallable = None) -> MetricCallable:
+        return self._metrics_implementations.get(metric_id, default_callable)
 
     def metric_class_by_id(self, metric_id: MetricsEnum) -> Metric:
         return self._metrics_implementations[metric_id].__self__()
