@@ -7,7 +7,7 @@ from .. import INSTANCE_OR_CALLABLE, Serializer
 
 def any_to_json(obj: INSTANCE_OR_CALLABLE) -> Dict[str, Any]:
     return {
-        **{k: v for k, v in vars(obj).items() if k != 'log'},
+        **{k: v for k, v in sorted(vars(obj).items()) if not _is_log_var(k)},
         **Serializer.dump_path_to_obj(obj)
     }
 
@@ -20,13 +20,18 @@ def any_from_json(cls: Type[INSTANCE_OR_CALLABLE], json_obj: Dict[str, Any]) ->
             for k, v in json_obj.items()
             if k in cls_parameters
         }
-        obj = cls(**init_data)
-        vars(obj).update({
+        vars_data = {
             k: json_obj[k]
             for k in json_obj.keys() ^ init_data.keys()
-        })
+        }
+        obj = cls(**init_data)
+        vars(obj).update(vars_data)
     else:
         init_data = deepcopy(json_obj)
         obj = cls(**init_data)
         vars(obj).update(json_obj)
     return obj
+
+
+def _is_log_var(varname: str) -> bool:
+    return varname.strip('_').startswith('log')
