@@ -1,15 +1,17 @@
 import os
 from dataclasses import dataclass
-from typing import List, Optional
+from typing import List, Optional, Sequence, Any, TypeVar, Callable, Hashable
 
 import numpy as np
 
+from fedot.core.dag.graph import Graph
+from fedot.core.dag.graph_verifier import GraphVerifier
 from fedot.core.data.data import InputData
 from fedot.core.log import default_log
-from fedot.core.pipelines.pipeline import Pipeline
-from fedot.core.pipelines.validation import validate
-from fedot.utilities.pattern_wrappers import singleton
+from fedot.core.pipelines.verification import verifier_for_task
+from fedot.core.utilities.serializable import Serializable
 from fedot.remote.infrastructure.clients.client import Client
+from fedot.utilities.pattern_wrappers import singleton
 
 
 def init_data_for_remote_execution(train_data: InputData):
@@ -20,7 +22,8 @@ def init_data_for_remote_execution(train_data: InputData):
 
 @dataclass
 class RemoteTaskParams:
-    """ Class with parameters of remote evaluation
+    """Class with parameters of remote evaluation.
+
     :param mode: evaluation mode - 'remote' or 'local'
     :param dataset_name: name of remote dataset used for fitting
     :param task_type: string representation of Task class for FEDOT
@@ -39,93 +42,109 @@ class RemoteTaskParams:
     max_parallel: int = 7
 
 
+G = TypeVar('G', bound=Serializable)
+
+
 @singleton
 class RemoteEvaluator:
     def __init__(self):
         """
         Class for the batch evaluation of pipelines using remote client
         """
-        self._logger = default_log('RemoteFitterLog')
+        self._logger = default_log(prefix='RemoteFitterLog')
         self.remote_task_params = None
         self.client = None
+        self.config_for_dump = _get_config
 
-    def init(self, client: Client = None, remote_task_params: Optional[RemoteTaskParams] = None):
+    def init(self, client: Client = None,
+             remote_task_params: Optional[RemoteTaskParams] = None,
+             get_config: Optional[Callable] = None):
         """
         :param client: client class for connection to external computational server.
         :param remote_task_params: dictionary with the parameters of remote evaluation.
+        :param get_config: optional function that constructs config for remote client.
 
         """
         self.remote_task_params = remote_task_params
         self.client = client
+        self.config_for_dump = get_config or _get_config
 
     @property
     def use_remote(self):
         return self.remote_task_params is not None and self.remote_task_params.mode == 'remote'
 
-    def compute_pipelines(self, pipelines: List['Pipeline']) -> List['Pipeline']:
+    def compute_graphs(self, graphs: Sequence[G], verifier: Optional[GraphVerifier] = None) -> Sequence[G]:
         params = self.remote_task_params
+        verifier = verifier or verifier_for_task()
 
         client = self.client
-        pipelines_parts = _prepare_batches(pipelines, params)
-        final_pipelines = []
+        execution_ids = {}
+        graph_batches = _prepare_batches(graphs, params.max_parallel)
+        final_graphs = []
 
         # start of the remote execution for each pipeline
-        for pipelines_part in pipelines_parts:
-            for pipeline in pipelines_part:
-                try:
-                    validate(pipeline)
-                except ValueError:
-                    pipeline.execution_id = None
-                    continue
-
-                pipeline_json, _ = pipeline.save()
-                pipeline_json = pipeline_json.replace('\n', '')
-
-                config = _get_config(pipeline_json, params, self.client.exec_params, self.client.connect_params)
-
-                task_id = client.create_task(config=config)
-
-                pipeline.execution_id = task_id
+        for graphs_batch in graph_batches:
+            for graph in graphs_batch:
+                task_id = self._create_graph_task(graph, verifier)
+                execution_ids[id(graph)] = task_id
 
             # waiting for readiness of all pipelines
             ex_time = client.wait_until_ready()
 
             # download of remote execution result for each pipeline
-            for p_id, pipeline in enumerate(pipelines_part):
-                if pipeline.execution_id:
+            for p_id, graph in enumerate(graphs_batch):
+                task_id = execution_ids.get(id(graph), None)
+                if task_id:
                     try:
-                        pipelines_part[p_id] = client.download_result(
-                            execution_id=pipeline.execution_id
-                        )
+                        graphs_batch[p_id] = client.download_result(task_id)
                     except Exception as ex:
-                        self._logger.warn(f'{p_id}, {ex}')
-            final_pipelines.extend(pipelines_part)
+                        self._logger.warning(f'{p_id}, {ex}')
+            final_graphs.extend(graphs_batch)
 
             self._logger.info(f'REMOTE EXECUTION TIME {ex_time}')
 
-        return final_pipelines
+        return final_graphs
+
+    def _create_graph_task(self, graph: G, verifier: Optional[GraphVerifier]) -> Optional[Hashable]:
+        """Serializes task and creates a graph task for remote client.
+        :return: task id
+        """
+        try:
+            if isinstance(graph, Graph) and verifier(graph):
+                pass
+        except ValueError:
+            return None
+
+        graph_json, _ = graph.save()
+        graph_json = graph_json.replace('\n', '')
+
+        config = self.config_for_dump(graph_json, self.remote_task_params,
+                                      self.client.exec_params, self.client.connect_params)
+
+        task_id = self.client.create_task(config=config)
+        return task_id
 
 
-def _prepare_batches(pipelines, params):
-    num_parts = np.floor(len(pipelines) / params.max_parallel)
+def _prepare_batches(graphs: Sequence[Any], max_parallel: int):
+    num_parts = np.floor(len(graphs) / max_parallel)
     num_parts = max(num_parts, 1)
-    pipelines_parts = [x.tolist() for x in np.array_split(pipelines, num_parts)]
+    pipelines_parts = [x.tolist() for x in np.array_split(graphs, num_parts)]
     return pipelines_parts
 
 
-def _get_config(pipeline_json, params: RemoteTaskParams, client_params: dict, conn_params: dict):
-    var_names = [str(name) for name in params.var_names] \
+def _get_config(graph_json: dict, params: RemoteTaskParams, client_params: dict, conn_params: dict):
+    var_names = list(map(str, params.var_names)) \
         if params.var_names is not None else []
-    train_data_idx = [str(idx) for idx in params.train_data_idx] \
+    train_data_idx = list(map(str, params.train_data_idx)) \
         if params.train_data_idx is not None else []
 
     data_name = params.dataset_name
-    if conn_params is not None and len(conn_params) > 0:
+    if conn_params is not None and conn_params:
         train_data = f"{client_params['container_input_path']}/data/{conn_params['DATA_ID']}/{data_name}.csv"
     else:
         train_data = f"{client_params['container_input_path']}/{data_name}.csv"
     return f"""[DEFAULT]
-        pipeline_template = {pipeline_json}
+        pipeline_template = {graph_json}
         train_data = {train_data}
         task = {params.task_type}
         output_path = {client_params['container_output_path']}
