@@ -9,33 +9,33 @@ from typing import List, Optional, Type, Union
 import matplotlib.pyplot as plt
 
 from fedot.core.data.data import InputData
-from fedot.core.log import Log, default_log
+from fedot.core.log import default_log
 from fedot.core.pipelines.node import Node
 from fedot.core.pipelines.pipeline import Pipeline
+from fedot.core.pipelines.verification import verify_pipeline
 from fedot.core.repository.operation_types_repository import OperationTypesRepository
 from fedot.core.utils import default_fedot_data_dir
-from fedot.core.pipelines.validation import validate
 from fedot.sensitivity.sa_requirements import ReplacementAnalysisMetaParams, SensitivityAnalysisRequirements
 from fedot.utilities.define_metric_by_task import MetricByTask
 
 
 class NodeAnalysis:
     """
-    :param approaches: methods applied to nodes to modify the pipeline or analyze certain operations.\
-    Default: [NodeDeletionAnalyze, NodeTuneAnalyze, NodeReplaceOperationAnalyze]
-    :param path_to_save: path to save results to. Default: ~home/Fedot/sensitivity
-    :param log: log: Log object to record messages
+    Args:
+        approaches: methods applied to nodes to modify the pipeline or analyze certain operations.
+            Default: [``NodeDeletionAnalyze``, ``NodeTuneAnalyze``, ``NodeReplaceOperationAnalyze``]
+        path_to_save: path to save results to. Default: ``~home/Fedot/sensitivity``
     """
 
     def __init__(self, approaches: Optional[List[Type['NodeAnalyzeApproach']]] = None,
                  approaches_requirements: SensitivityAnalysisRequirements = None,
-                 path_to_save=None, log: Log = None):
+                 path_to_save=None):
 
         self.approaches = [NodeDeletionAnalyze, NodeReplaceOperationAnalyze] if approaches is None else approaches
 
         self.path_to_save = \
             join(default_fedot_data_dir(), 'sensitivity', 'nodes_sensitivity') if path_to_save is None else path_to_save
-        self.log = default_log(__name__) if log is None else log
+        self.log = default_log(self)
 
         self.approaches_requirements = \
             SensitivityAnalysisRequirements() if approaches_requirements is None else approaches_requirements
@@ -44,15 +44,17 @@ class NodeAnalysis:
                 train_data: InputData, test_data: InputData,
                 is_save: bool = False) -> dict:
 
-        """
-        Method runs Node analysis within defined approaches
+        """Method runs Node analysis within defined approaches
 
-        :param is_save: whether the certain node analysis result is needed to ba saved
-        :param pipeline: Pipeline containing the analyzed Node
-        :param node: Node object to analyze in Pipeline
-        :param train_data: data used for Pipeline training
-        :param test_data: data used for Pipeline validation
-        :return: dict with Node analysis result per approach
+        Args:
+            is_save: whether the certain node analysis result is needed to ba saved
+            pipeline: :obj:`Pipeline` containing the analyzed :obj:`Node`
+            node: :obj:`Node` object to analyze in :obj:`Pipeline`
+            train_data: data used for :obj:`Pipeline` training
+            test_data: data used for :obj:`Pipeline` validation
+
+        Returns:
+            dict:  :obj:`Node` analysis result per approach
         """
 
         results = dict()
@@ -115,23 +117,22 @@ class NodeAnalysis:
         with open(result_file, 'w', encoding='utf-8') as file:
             file.write(json.dumps(results, indent=4))
 
-        self.log.message(f'Node Sensitivity analysis results were saved to {result_file}')
+        self.log.info(f'Node Sensitivity analysis results were saved to {result_file}')
 
 
 class NodeAnalyzeApproach(ABC):
-    """
-    Base class for analysis approach.
+    """Base class for analysis approach.
 
-    :param pipeline: Pipeline containing the analyzed Node
-    :param train_data: data used for Pipeline training
-    :param test_data: data used for Pipeline validation
-    :param path_to_save: path to save results to. Default: ~home/Fedot/sensitivity
-    :param log: log: Log object to record messages
+    Args:
+        pipeline: :obj:`Pipeline` containing the analyzed :obj:`Node`
+        train_data: data used for :obj:`Pipeline` training
+        test_data: data used for :obj:`Pipeline` validation
+        path_to_save: path to save results to. Default: ``~home/Fedot/sensitivity``
     """
 
     def __init__(self, pipeline: Pipeline, train_data, test_data: InputData,
                  requirements: SensitivityAnalysisRequirements = None,
-                 path_to_save=None, log: Log = None):
+                 path_to_save=None):
         self._pipeline = pipeline
         self._train_data = train_data
         self._test_data = test_data
@@ -141,7 +142,7 @@ class NodeAnalyzeApproach(ABC):
 
         self._path_to_save = \
             join(default_fedot_data_dir(), 'sensitivity', 'nodes_sensitivity') if path_to_save is None else path_to_save
-        self.log = default_log(__name__) if log is None else log
+        self.log = default_log(self)
 
         if not exists(self._path_to_save):
             makedirs(self._path_to_save)
@@ -151,7 +152,8 @@ class NodeAnalyzeApproach(ABC):
         """Creates the difference metric(scorer, index, etc) of the changed
         graph in relation to the original one
 
-        :param node: the sequence number of the node as in DFS result
+        Args:
+            node: the sequence number of the node as in ``DFS`` result
         """
         pass
 
@@ -171,7 +173,7 @@ class NodeAnalyzeApproach(ABC):
         return changed_pipeline_metric / self._origin_metric
 
     def _get_metric_value(self, pipeline: Pipeline, metric: MetricByTask) -> float:
-        pipeline.fit(self._train_data, use_fitted=False)
+        pipeline.fit(self._train_data)
         predicted = pipeline.predict(self._test_data)
         metric_value = metric.get_value(true=self._test_data,
                                         predicted=predicted)
@@ -187,9 +189,13 @@ class NodeDeletionAnalyze(NodeAnalyzeApproach):
 
     def analyze(self, node: Node, **kwargs) -> Union[List[dict], List[float]]:
         """
-        :param node: Node object to analyze
-        :return: the ratio of modified pipeline score to origin score
+        Args:
+            node: :obj:`Node` object to analyze
+
+        Returns:
+            the ratio of modified pipeline score to origin score
         """
+
         if node is self._pipeline.root_node:
             # TODO or warning?
             return [1.0]
@@ -205,18 +211,20 @@ class NodeDeletionAnalyze(NodeAnalyzeApproach):
 
     def sample(self, node: Node):
         """
-
-        :param node: Node object to delete from Pipeline object
-        :return: Pipeline object without node
+        Args:
+            node: :obj:`Node` object to delete from :obj:`Pipeline` object
+        Retuens:
+            :obj:`Pipeline`: pipeline without node
         """
+
         pipeline_sample = deepcopy(self._pipeline)
         node_index_to_delete = self._pipeline.nodes.index(node)
         node_to_delete = pipeline_sample.nodes[node_index_to_delete]
         pipeline_sample.delete_node(node_to_delete)
         try:
-            validate(pipeline_sample)
+            verify_pipeline(pipeline_sample)
         except ValueError as ex:
-            self.log.message(f'Can not delete node. Deletion of this node leads to {ex}')
+            self.log.info(f'Can not delete node. Deletion of this node leads to {ex}')
             return None
 
         return pipeline_sample
@@ -226,9 +234,7 @@ class NodeDeletionAnalyze(NodeAnalyzeApproach):
 
 
 class NodeReplaceOperationAnalyze(NodeAnalyzeApproach):
-    """
-    Replace node with operations available for the current task
-    and evaluate the score difference
+    """Replace node with operations available for the current task and evaluate the score difference
     """
 
     def __init__(self, pipeline: Pipeline, train_data: InputData, test_data: InputData,
@@ -238,10 +244,13 @@ class NodeReplaceOperationAnalyze(NodeAnalyzeApproach):
 
     def analyze(self, node: Node, **kwargs) -> Union[List[dict], List[float]]:
         """
-        :param node: Node object to analyze
+        Args:
+            node: :obj:`Node` object to analyze
 
-        :return: the ratio of modified pipeline score to origin score
+        Returns:
+            the ratio of modified pipeline score to origin score
         """
+
         requirements: ReplacementAnalysisMetaParams = self._requirements.replacement_meta
         node_id = self._pipeline.nodes.index(node)
         samples = self.sample(node=node,
@@ -268,12 +277,13 @@ class NodeReplaceOperationAnalyze(NodeAnalyzeApproach):
                nodes_to_replace_to: Optional[List[Node]],
                number_of_random_operations: Optional[int] = None) -> Union[List[Pipeline], Pipeline]:
         """
-
-        :param node: Node object to replace
-        :param nodes_to_replace_to: nodes provided for old_node replacement
-        :param number_of_random_operations: number of replacement operations, \
-        if nodes_to_replace_to not provided
-        :return: Sequence of Pipeline objects with new operations instead of old one.
+        Args:
+            node: :obj:`Node` object to replace
+            nodes_to_replace_to: nodes provided for old_node replacement
+            number_of_random_operations: number of replacement operations,
+                if ``nodes_to_replace_to`` not provided
+        Returns:
+            Union[List[Pipeline], Pipeline]: sequence of :obj:`Pipeline` objects with new operations instead of old one
         """
 
         if not nodes_to_replace_to:
@@ -298,7 +308,7 @@ class NodeReplaceOperationAnalyze(NodeAnalyzeApproach):
         # Get models
         app_models, _ = OperationTypesRepository().suitable_operation(task_type=task)
         # Get data operations for such task
-        app_data_operations, _ = OperationTypesRepository('data_operation').suitable_operation(
+        app_data_operations = OperationTypesRepository('data_operation').suitable_operation(
             task_type=task)
 
         # Unit two lists
@@ -337,7 +347,7 @@ class NodeReplaceOperationAnalyze(NodeAnalyzeApproach):
         file_name = f'{self._pipeline.nodes[node_id].operation.operation_type}_id_{node_id}_replacement.jpg'
         result_file = join(self._path_to_save, file_name)
         plt.savefig(result_file)
-        self.log.message(f'NodeReplacementAnalysis for '
+        self.log.info(f'NodeReplacementAnalysis for '
                          f'{original_operation_type}(index:{node_id}) was saved to {result_file}')
 
     def __str__(self):
