@@ -1,42 +1,40 @@
 import json
 from os.path import join
-from typing import Optional, List, Type
+from typing import List, Optional, Type
 
 import numpy as np
 from matplotlib import pyplot as plt
 
-from fedot.core.pipelines.pipeline import Pipeline
-from fedot.core.pipelines.node import Node
 from fedot.core.dag.graph_operator import GraphOperator
 from fedot.core.data.data import InputData
-from fedot.core.log import Log, default_log
+from fedot.core.log import default_log
+from fedot.core.pipelines.node import Node
+from fedot.core.pipelines.pipeline import Pipeline
 from fedot.core.utils import default_fedot_data_dir
-from fedot.sensitivity.node_sa_approaches import NodeAnalyzeApproach, NodeAnalysis
+from fedot.sensitivity.node_sa_approaches import NodeAnalysis, NodeAnalyzeApproach
 from fedot.sensitivity.sa_requirements import SensitivityAnalysisRequirements
 
 
 class NodesAnalysis:
-    """
-    This class is for nodes sensitivity analysis within a Pipeline .
-    It takes nodes and approaches to be applied to chosen nodes.
-    To define which nodes to analyze pass them to nodes_to_analyze filed
+    """This class is for nodes sensitivity analysis within a :obj:`Pipeline`\n
+    It takes nodes and approaches to be applied to chosen nodes.\n
+    To define which nodes to analyze pass them to ``nodes_to_analyze`` filed
     or all nodes will be analyzed.
 
-    :param pipeline: pipeline object to analyze
-    :param train_data: data used for Pipeline training
-    :param test_data: data used for Pipeline validation
-    :param approaches: methods applied to nodes to modify the pipeline or analyze certain operations.\
-    Default: [NodeDeletionAnalyze, NodeReplaceOperationAnalyze]
-    :param nodes_to_analyze: nodes to analyze. Default: all nodes
-    :param path_to_save: path to save results to. Default: ~home/Fedot/sensitivity
-    :param log: log: Log object to record messages
+    Args:
+        pipeline: pipeline object to analyze
+        train_data: data used for Pipeline training
+        test_data: data used for Pipeline validation
+        approaches: methods applied to nodes to modify the pipeline or analyze certain operations.\n
+            Default: [:obj:`NodeDeletionAnalyze`, :obj:`NodeReplaceOperationAnalyze`]
+        nodes_to_analyze: nodes to analyze. Default: all nodes
+        path_to_save: path to save results to. Default: ``~home/Fedot/sensitivity``
     """
 
     def __init__(self, pipeline: Pipeline, train_data: InputData, test_data: InputData,
                  approaches: Optional[List[Type[NodeAnalyzeApproach]]] = None,
                  requirements: SensitivityAnalysisRequirements = None,
-                 path_to_save=None, log: Log = None,
-                 nodes_to_analyze: List[Node] = None):
+                 path_to_save=None, nodes_to_analyze: List[Node] = None):
 
         self.pipeline = pipeline
         self.train_data = train_data
@@ -45,21 +43,21 @@ class NodesAnalysis:
         self.requirements = \
             SensitivityAnalysisRequirements() if requirements is None else requirements
         self.metric = self.requirements.metric
-        self.log = default_log(__name__) if log is None else log
+        self.log = default_log(self)
         self.path_to_save = \
             join(default_fedot_data_dir(), 'sensitivity', 'nodes_sensitivity') if path_to_save is None else path_to_save
 
         if not nodes_to_analyze:
-            self.log.message('Nodes to analyze are not defined. All nodes will be analyzed.')
+            self.log.info('Nodes to analyze are not defined. All nodes will be analyzed.')
             self.nodes_to_analyze = self.pipeline.nodes
         else:
             self.nodes_to_analyze = nodes_to_analyze
 
     def analyze(self) -> dict:
-        """
-        Main method to run the analyze process for every node.
+        """Main method to run the analyze process for every node.
 
-        :return nodes_results: dict with analysis result per Node
+        Returns:
+            dict: with analysis result per Node
         """
 
         nodes_results = dict()
@@ -91,7 +89,7 @@ class NodesAnalysis:
         with open(result_file, 'w', encoding='utf-8') as file:
             file.write(json.dumps(result, indent=4))
 
-        self.log.message(f'Pipeline Sensitivity Analysis results were saved to {result_file}')
+        self.log.info(f'Pipeline Sensitivity Analysis results were saved to {result_file}')
 
     def _visualize_result_per_approach(self, results: dict, types: list):
         gathered_results = self._extract_result_values(results)
@@ -111,10 +109,10 @@ class NodesAnalysis:
                              f'{self.approaches[index].__name__}.jpg')
 
             plt.savefig(file_path)
-            self.log.message(f'Pipeline Sensitivity Analysis visualized results were saved to {file_path}')
+            self.log.info(f'Pipeline Sensitivity Analysis visualized results were saved to {file_path}')
 
     def _visualize_degree_correlation(self, results: dict):
-        nodes_degrees = GraphOperator(self.pipeline).get_nodes_degrees()
+        nodes_degrees = self.pipeline.operator.get_nodes_degrees()  # pipeline.operator is GraphOperator
         gathered_results = self._extract_result_values(results)
         for index, result in enumerate(gathered_results):
             fig, ax = plt.subplots(figsize=(15, 10))
@@ -123,7 +121,7 @@ class NodesAnalysis:
             file_path = join(self.path_to_save,
                              f'{self.approaches[index].__name__}_cor.jpg')
             plt.savefig(file_path)
-            self.log.message(f'Nodes degree correlation visualized results were saved to {file_path}')
+            self.log.info(f'Nodes degree correlation visualized results were saved to {file_path}')
 
     def _extract_result_values(self, results):
         gathered_results = []
